'Antikythera Calculator for Propeller:  Calculate the phase of the moon like the ancient Greeks!
'The Antikythera device used gears to calculate the phase of the moon
'We do the same calculation here, but with integer math
'Result is accurate to 1 day in 500 years!
'Uses portions of real-time clock driver "DS1339_Driver.Spin" to calculate the day of the year
'Copyright 2011 Raymond Allen, rayslogic.com, LLC
'Default is for TV display, but easily changed to VGA as noted below

CON   'Enter day of interest here!  Example is for 07 February 2011
  year = 2011
  month = 2
  dayofmonth = 7 

CON  'Constants Section

  _clkmode = xtal1 + pll16x      'using 5 MHz crystal in pll16x mode
  _xinfreq = 5_000_000           'to achieve 80 Mhz clock frequency 


OBJ  'Objects Section
    'We need the TV (or VGA) text driver
    
  text :  "tv_text"   'Note: For VGA output: Comment out this line...
  text1 : "VGA_text"  'and remove "1" from "text1"  (need to change pin parameter for text.start(pin) command below too).


VAR    'just need a couple working variables
  long callipic, callipics, julian, leaps, days, c, phase, percent  



PUB Antikythera    'Calculate the Phase of the Moon like the ancient Greeks!
  
  text.start(12)    'Start the VGA/TV text driver (uses another cog)
                    'The parameter (12) is the base pin used by demo and proto boards for TV output
                    'Change (12) to (16) when using "VGA_text" driver with demo board as it uses pin#16 for vga

  text.str(string("Antikythera Simulator",13,13))
  text.str(string("Calculating Moon Phase for:",13,"  "))
  text.dec(dayofmonth)
  text.out("/")
  text.dec(month)
  text.out("/")
  text.dec(year) 
  

  julian:=DayOfYear(year,month-1,dayofmonth)
  julian++  'Julian is now the day of the year [1..366]

  text.str(string(13,13, "Days into year:",13," "))
  text.dec(julian)
  
  'First, figure out where we are on the callipic cycle, a 76 year repeating cycle
  callipic:=year//76
  callipics:=year/76

  'Account for leap years
  leaps:=0
  if (callipic>0)
    leaps:=(callipic-1)/4

  'calculate actual days into 76-year cycle cycle    
  days:=callipic*365+julian+leaps-callipics


  text.str(string(13,13,"Days into 76 year cycle:",13," "))
  text.dec(days)

  
  c:=1222    'c is an offset accounting for moon phase at day one
  'The callipic cycle is 940 lunations in 27,759 days
  'It may be more accurate to multiply by 940 and divide by 27,759
  'Instead we multiply by 47 and divide by 1388 (requires less gear teeth/bits)
  phase:=(days*47+c)//1388  '==931 on 10Dec06

  'Scale moon phase to 100%
  percent:=phase*100/1388
  'cycle starts and ends with new moon.  So, 0% and 99% are new moon, 50% is full moon.

  text.str(string(13,13,"Percent of Moon Cycle: "))
  text.dec(percent)
  text.out("%")  

  case percent
    0..3:
      text.str(string(13,"> New Moon"))
    4..21:
      text.str(string(13,"> Waxing Cresent"))
    22..28:
      text.str(string(13,"> First Quarter"))      
    29..46:
      text.str(string(13,"> Waxing Gibbous"))
    47..53:
      text.str(string(13,"> Full Moon"))
    54..71:
      text.str(string(13,"> Waning Gibbous"))
    72..78:
      text.str(string(13,"> Last Quarter"))
    79..96:
      text.str(string(13,"> Waning Cresent"))
    97..99:
      text.str(string(13,"> New Moon")) 
                  
  repeat


PUB DayOfYear(yr,mon,mday):yday
  'Gets the returns day of the year [0..365]
  if (NOT IsLeap(yr))
    'normal year
     yday:=word[@NormDaysBeforeMon][mon]
  else
    'leap year
     yday:=word[@LeapDaysBeforeMon][mon]
  yday+=mday-1
  return yday   

PUB IsLeap(yr):bLeap
  'Determines if year (ordinary integer, e.g., 2007, is a leap year or not
  'a year is leap if it is divisible by 4, unless it is divisible by 100, except when divisible by 400
  'So, 1900 is not a leap year, but 2000 is...
  return ((yr// 4 == 0) AND ((yr//100) <> 0)) OR ((yr//400) == 0)  

DAT
'Table of # days that come before each month (0-12)
NormDaysBeforeMon word  0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334, 365   ' Normal years
LeapDaysBeforeMon word  0, 31, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335, 366   ' Leap years
